/* rtn.h  acb  8-9-1994 
 *
 * Major modifications:
 * 17-9-1994: started adding parametrised rules
 * 23-2-1994 (?): started adding regular expression-based mappings
 * 2[56]-9-1994: started adding dereferencing
 * 15-2-1994: Started adding embedded imperative code
 */

#ifndef __RTN_H
#define __RTN_H

#include "ptrlist.h"

enum nodetype { 
    literal, 
    symbol, 
    mapping, 
    deref, 
    var_conddef,  /* conditional variable definition; define if not already 
		     so */
    var_def,      /* force definition of a variable */
    var_ref,      /* reference a variable */
    code,         /* a chunk of code which is executed */
    silence       /* something which silences another node. */
};

typedef struct tagNode {
  enum nodetype type;
  char *data; /* literal text, symbol invocation, mapping invocation, 
		 block of code or name of variable to set or retrieve */
  struct tagNode *params; /* actual parameters, to be evaluated, or subnode 
			     for mapping or conditional definition */
  struct tagNode *next;
} Node, *pNode;

typedef struct tagOption {
  pNode atoms;
  struct tagOption *next;
} Option, *pOption;

/* parameters for parametrised rules */

typedef ListNode Param, *pParam;

/* rules are arranged in a binary tree */

typedef struct tagRule {
  char *symbol;
  pOption options;
  pParam params;  /* parameters for this rule, or NULL if none are used, */
  struct tagRule *left, *right;
  /* added on 07/01/25 AU; a sort of limited memory to counteract seriality
     somewhat */
  int last_choice;
} Rule, *pRule;

/* the second parameter is a long, to accomodate 64-bit pointers on Alphas. */

typedef int (*NodeIterator)(pNode, long);

typedef int (*OptionIterator)(pOption, long);

/*typedef int (*ParamIterator)(pParam, long);*/

typedef ListIterator ParamIterator;

typedef int (*RuleIterator)(pRule, long);

/*
 * function prototypes
 */

pNode node_cons(enum nodetype t, char *d, pNode cdr);

pNode node_append(pNode a, pNode b);

void node_map(pNode list, NodeIterator iter, long param);

pOption option_cons(pNode car, pOption cdr);

pOption option_append(pOption a, pOption b);

int option_length(pOption list);

pOption option_nth(pOption list, int index);

void option_map(pOption list, OptionIterator iter, long param);

int param_indexof(pListNode list, char *nm);

/* make a new rule, sans subtrees */
pRule rule_new(char *symbol, pOption options, pParam params);

/* a recursive insertion function */

pRule rule_insert(pRule parent, pRule newrule);

/* find a rule */

pRule rule_find(pRule tree, char *name);

/* traverse the rule tree in infix order */

void rule_inorder_traverse(pRule tree, RuleIterator iter, long param);

#endif
